/********************************************************************/
/*	EASYFSEL.C														*/
/*																	*/
/*	Routine zum leichten und AES-Versions abhngigen Aufruf der		*/
/*	GEM-Fileselectorbox mit Auswertung des 'FSEL' - Cookies.		*/
/*	Auerdem sind einige Routinen zum Umgang mit Dateien enthalten,	*/
/*	sowie zum Suchen eines Cookies.									*/
/*																	*/
/*	Version : 1.27													*/
/*	Datum   : 15.06.1992											*/
/*	Autor		: Andreas Papula (falls nicht anders angegeben)		*/
/*																	*/
/*	Copyright 1992 by MAXON Computer GmbH							*/
/********************************************************************/

/*------------------------------------------------------------------*/
/*	Include-Files einbinden.										*/
/*------------------------------------------------------------------*/

#include <easyfsel.h>
#include <aes.h>
#include <ctype.h>
#include <stddef.h>
#include <ext.h>
#include <my_ext.h>
#include <string.h>
#include <stdlib.h>

/* -------------------------------------------------------------------- */
/*       Makros. & Konstanten                                           */
/* -------------------------------------------------------------------- */

#define _sysbase	(0x4f2l)

/* -------------------------------------------------------------------- */
/*		typedefs														*/
/* -------------------------------------------------------------------- */

/* Erweitertes DTA fr lange Filenamen	*/
typedef struct
{
    char            d_reserved[21];
    unsigned char   d_attrib;
    unsigned int    d_time;
    unsigned int    d_date;
    unsigned long   d_length;
    int				d_max_pfad;		/* Max Lnge des Pfades	*/
    int				d_max_name;		/* Max Lnge des Namen	*/
    char            *d_fname;
	char			*d_pfad;
	char			d_maske[64];
	long			d_handle;
	int				d_flags;		/* fsfirst etc verwenden s.u.*/
	int				d_sattr;		/* Die Fsfirst Attribut		*/
	int				d_case_s;		/* Case sensitiv? s.u.		*/
} DTAX;

/* -------------------------------------------------------------------- */
/*		Globale Variablen												*/
/* -------------------------------------------------------------------- */

DTA Dta;
DTAX My_dta;
char *Tmpdir;
/* -------------------------------------------------------------------- */
/* 		lokale Funktionsprototypen										*/
/* -------------------------------------------------------------------- */
/*DYJ 2.03b Start*/
/*DYJ  Ende; alt:	
long * get_cookie_ptr(long cookie_name);
*/
/*------------------------------------------------------------------*/
/*	BOOLEAN easy_fsel(BYTE *pfad, BYTE *dateiname, BYTE *text)		*/
/*																	*/
/*	Zeigt die Fileselectorbox je nach AES-Version und 'FSEL'-Cookie	*/
/*	an und bernimmt die Auswertung.								*/
/*	Bei einer AES-Version >= 0x0014 oder gesetztem 'FSEL-Cookie'	*/
/*	wird fsel_exinput() aufgerufen, sonst fsel_input().				*/
/*																	*/
/*	Parameter:Pfad(incl. Wildcard) und Dateiname, mit denen die Fileselectorbox 	*/
/*						aufgerufen werden soll.						*/
/*	Rckgabe :TRUE, wenn alles glattging und der Benutzer OK ge-	*/
/*				drckt hat, FALSE falls ein Fehler aufgetreten ist	*/
/*				oder Abbruch gewhlt wurde.	Der neue Pfad steht		*/
/*				in 'pfad', der ausgewhlte Dateiname in dateiname'.	*/
/*------------------------------------------------------------------*/

BOOLEAN easy_fsel(BYTE *pfad, BYTE *dateiname, BYTE *text)
{
	WORD button;
	WORD result;
	LONG c_wert = 0;
	char *blank;
/*	extern GEMPARBLK _GemParBlk;*/
	/* DYJ ' ' im Pfad- und Dateinamen eliminieren*/
	if((blank=strchr(pfad,' '))!=NULL)
		*blank=0;
	if((blank=strchr(dateiname,' '))!=NULL)
		*blank=0;
	/* Entsprechend der Version und des 'FSEL' - Cookies */
	/* fsel_input() oder fsel_exinput() aufrufen */

	if(_GemParBlk.global[0] < 0x0140 && get_cookie('FSEL', &c_wert) == FALSE)
		result = fsel_input(pfad, dateiname, &button);
	else
		result = fsel_exinput(pfad, dateiname, &button, text);

	/* Wenn Fehler aufgetreten oder Abbruch ausgewhlt */
	if(result == 0 || button == 0)
		return FALSE;
	else
		return TRUE;
}

/*------------------------------------------------------------------*/
/*	VOID build_filename(BYTE *dest, BYTE *pfad, BYTE *dateiname,BOOLEAN noFinPf)	*/
/*																	*/
/*	Bastelt Pfad- und Dateinamen zusammen.							*/
/*																	*/
/*	Parameter:	dest      = Zielstring.								*/
/*				pfad      = Pfadname.								*/
/*				dateiname = Dateiname.								*/
/*				noFinPf:  =no File in Pfadname						*/
/* bisher ging build_filename davon aus, da evtl. einen Pfadnamen	*/
/*	enthlt, der dann verworfen wird. Dies wird jetzt mit 			*/
/*	noFinPf=FALSE erreicht. Sonst wird immer an pfad angehngt,		*/
/*	ggf auch zuerst ein '\\'										*/
/*	Rckgabe :Das Ergebnis befindet sich in dest.					*/
/*------------------------------------------------------------------*/

VOID build_filename(BYTE *dest, BYTE *pfad, BYTE *dateiname,BOOLEAN noFinPf)
{
	BYTE *xyz;
	size_t len;

	strcpy(dest, pfad);
	if(noFinPf &&*dest)
	{
		len=strlen(dest);
		if(dest[len-1]!='\\')
			strcat(dest,"\\");
	}
	xyz = strrchr(dest, (int) '\\');
	strcpy(++xyz, dateiname);
}

/*------------------------------------------------------------------*/
/*	BOOLEAN exist(const BYTE *dateiname,int case_ign)				*/
/*	BOOLEAN exist_dir(const BYTE *dirname,int case_ign)				*/
/*																	*/
/*	berprft, ob die Datei 'dateiname' existiert.					*/
/*																	*/
/*	Parameter:	dateiname,dirname:	Name der Datei/Dir				*/
/*				case_ign: ob Gro/Klleinschr. ignor. werden soll	*/
/*	Rckgabe :TRUE, falls die Datei existiert, FALSE wenn nicht.	*/
/*------------------------------------------------------------------*/

BOOLEAN exist(const BYTE *dateiname,int case_ign)
{
	if(!dateiname || !*dateiname)
		return FALSE;
	if(fsfirst(dateiname, FA_READONLY | FA_HIDDEN | FA_SYSTEM |
					FA_ARCHIVE,case_ign) == 0)
		return TRUE;
	else
		return FALSE;
}
BOOLEAN exist_dir(const BYTE *dirname,int case_ign)
{
	char dirName[PATH_MAX],*ptr;
	strcpy(dirName,dirname);
	ptr=strrchr(dirName,'\\');
	if(ptr && !ptr[1])
		*ptr='\0';
	if(fsfirst(dirName, FA_SUBDIR,case_ign) == 0)
		return TRUE;
	else
		return FALSE;
}
/*** ------------------------------------------------------------------------ ***/
/*	BOOLEAN exist_dirOfFile(const BYTE *file,int case_ign)						*/
/*	Prft ob der Ordner fr ein File existiert									*/
/*	Parameter:	file:	das File												*/
/*				case_ign Gro/Kleinschr. ignorieren?							*/
/*	Ergebnis:	TRUE wenn vorhanden												*/		
/*	25.11.02 von Dimitri Junker	DYJ 1.46b										*/
/*** ------------------------------------------------------------------------ ***/
BOOLEAN exist_dirOfFile(const BYTE *file,int case_ign)
{
	char pfad[PATH_MAX],name[PATH_MAX];
	trenn_pfad(pfad,name,file);
	return exist_dir(pfad,case_ign);
}
/*------------------------------------------------------------------*/
/*			BYTE *get_akt_path(BYTE *path)							*/
/*																	*/
/*			Gibt das aktuelle Laufwerk und dessen Pfad zurck.		*/
/*																	*/
/*		Parameter: Zeiger auf einen String.							*/
/*		Rckgabe : Zeiger auf das aktuelle Laufwerk und Directory.	*/
/*------------------------------------------------------------------*/

BYTE *get_akt_path(BYTE *path)
{
	strcpy(path, " :");
	path[0] = 'A' + getdisk();  /* Laufwerk */
	getcurdir(0, path+2);				/* Directory */
	strcat(path,"\\");
	return(path);
}
/*------------------------------------------------------------------*/
/*			void set_akt_path(BYTE *path)							*/
/*																	*/
/*			Setzt das aktuelle Laufwerk und den Pfad.				*/
/*																	*/
/*			Parameter: Zeiger auf einen String						*/
/*				(Pfad mit oder ohne File.							*/
/*------------------------------------------------------------------*/

void set_akt_path(BYTE *path)
{	int disk,pfad_ab=0;
	char sich,*null;
	if(path[1]==':')
	{	if(path[0]>='A'&&path[0]<='Z')
			disk=path[0]-'A';
		else
			disk=path[0]-'a';
		setdisk(disk);
		pfad_ab=2;
	}
	null=strrchr(path,'\\')+1;
	sich=*null;
	*null='\0';
	chdir(&path[pfad_ab]);
	*null=sich;
	return;
}

/*------------------------------------------------------------------*/
/*	void trenn_pfad(char *pfad,char *name,char *file)				*/
/*																	*/
/*	Bastelt Pfad- und Dateinamen auseinander, 						*/
/*	Gegenteil von build_filename.									*/
/*	dabei endet der Pfad mit '\\'									*/
/*																	*/
/*	Parameter:			file	= File mit Pfad+Name				*/
/*						pfad    = Pfadname.							*/
/*						name 	= Dateiname.						*/
/*			file und pfad drfen gleich sein						*/
/*	war nicht Teil des Originals, also von DYJ						*/
/*------------------------------------------------------------------*/

void trenn_pfad(char *pfad,char *name,const char *file)
{ 	char *ptr;
	if(file!=pfad)
		strcpy(pfad,file);
	if((ptr=strrchr(pfad,'\\'))==NULL)
		ptr=pfad;					/* dann ist wohl alles nur Name	*/
	else
		ptr++;
	strcpy(name,ptr);
	*ptr='\0';
}
	

/*------------------------------------------------------------------*/
/*	BOOLEAN get_cookie(BYTE *cookie_name, LONG *cookie_value)		*/
/*																	*/
/*	Sucht nach einem Cookie 'cookie_name' und liefert dessen Wert	*/
/*	in 'cookie_value' zurck.										*/
/*																	*/
/*	Parameter:Name des Cookies und einen Zeiger auf den Wert des	*/
/*						Cookies.									*/
/*	Rckgabe :TRUE, falls der Cookie gefunden wurde, FALSE falls	*/
/*						nicht.										*/
/*					In 'cookie_value' steht der Wert des Cookies.	*/
/*					cookie_value kann NULL sein, dann wird nur die	*/
/*					Existenz des Cookies getestet					*/
/*------------------------------------------------------------------*/
/*DYJ 2.03b Start*/
BOOLEAN get_cookie( LONG cookie, LONG *value )
{
  static WORD use_ssystem = -1;
  COOKJAR *cookiejar;
  LONG    val = -1l;
  WORD    i=0;

  if( use_ssystem < 0 )
    use_ssystem = (Ssystem(S_INQUIRE, 0l, 0)==E_OK);

  if(use_ssystem)
  {
    if( Ssystem(S_GETCOOKIE, cookie, (LONG)&val)==E_OK )
    {
      if( value!=NULL )
        *(LONG *)value = val;
      return TRUE;
    }
  }
  else
  {
    /* Zeiger auf Cookie Jar holen */
    cookiejar = (COOKJAR *)(Setexc(0x05A0/4,(void (*)(void))-1));
    if( cookiejar )
    {
      for( i=0 ; cookiejar[i].id ; i++ )
        if( cookiejar[i].id==cookie )
        {
          if( value )
            *(LONG *)value = cookiejar[i].value;
          return TRUE;
        }
    }
  }

  return FALSE;
}
/*DYJ  Ende; alt:	

BOOLEAN get_cookie(long cookie_name, LONG *cookie_value)
{
	LONG *cookie;
	int ret=1;
	
	/* Zuerst einen Zeiger auf den Cookie holen */

	cookie = get_cookie_ptr(cookie_name);

	/* Ist der Cookiejar vorhanden ? */
    
	if(!cookie)
		ret=0;
	else
	{	if(!cookie[0])
			ret=0;
		else
		{	if(cookie_value)
				*cookie_value=cookie[1];
		}
	}
	return(ret);
}
/*------------------------------------------------------------------*/
/*	long get_cookie_jar()											*/
/*																	*/
/*	Ermittelt die Adresse des Cookie Jar.							*/
/*																	*/
/*	Parameter:keine				 									*/
/*	Rckgabe :Adresse, falls der Cookie Jar gefunden wurde,			*/
/*			  NULL falls kein Cookie-Jar							*/
/*  Besonderheit: mu per Supexec aufgerufen werden					*/
/*	(c) by Dimitri Junker 1996										*/
/*------------------------------------------------------------------*/

long  get_cookie_jar()
{
	LONG *cookiejar;

	cookiejar = *((LONG **) 0x5a0L);
	return((long)cookiejar);
}
/*------------------------------------------------------------------*/
/*	long *get_cookie_ptr(BYTE *cookie_name)							*/
/*																	*/
/*	Sucht nach einem Cookie 'cookie_name' und liefert dessen 		*/
/*	Adresse zurck.													*/
/*																	*/
/*	Parameter:Name des Cookies 										*/
/*	Rckgabe :Adresse, falls der Cookie gefunden wurde,				*/
/*			  Adresse des NULL-cookie falls nicht vorhanden			*/
/*			  NULL falls kein Cookie-Jar							*/
/*	(c) by Dimitri Junker 1996										*/
/*------------------------------------------------------------------*/

long * get_cookie_ptr(long cookie_name)
{
	LONG *cookiejar;

	/* Zuerst einen Zeiger auf den Cookiejar holen */

	cookiejar= (long *)Supexec(get_cookie_jar);

	/* Ist der Cookiejar vorhanden ? */
    
	if(!cookiejar)
		return(NULL);
	else
	{
		do
		{
			/* Ist es unser Cookie ? */
			if(*cookiejar== cookie_name)
				break;
			else
			{
				/* nchster Cookie aus Jar */
				cookiejar = &(cookiejar[2]);
			}
		} 
		while(cookiejar[0]);  /* solange nicht NULL-Cookie */
    
		return(cookiejar);;
  }
}
*/

/*------------------------------------------------------------------*/
/*	int fsfirst( const char *filename, int attr,int case_ign );		*/
/*																	*/
/*	Wie das groe Vorbild, aber auch fr alternative Filesysteme	*/
/*																	*/
/*	Parameter:filename: Filename der gesucht werden soll				*/
/*			  attr: s. Fsfirst										*/
/*			  ign_case:	ungeachtet der Casesensitivitt des Datei-	*/
/*				systems soll Gro-/Kleinschreibung ignoriert werden	*/
/*	Rckgabe :Die Funktion liefert als Ergebnis 0, wenn eine Datei	*/
/*              gefunden werden konnte, ansonsten einen Fehlercode.	*/
/*			die eigentliche Rckgabe geschiet in My_dta				*/
/*	(c) by Dimitri Junker 1996										*/
/*------------------------------------------------------------------*/

int fsfirst( const char *filename, int attr ,int case_ign )
{	static int first=1;
	int rueck;
	XATTR xdummy;
	long ldummy;
	if(first)
	{	first=0;
		Fsetdta(&Dta);
		My_dta.d_flags=0;
		My_dta.d_max_name=FILENAME_MAX;
		My_dta.d_max_pfad=PATH_MAX;
		My_dta.d_fname=malloc(My_dta.d_max_name+My_dta.d_max_pfad+4);
		My_dta.d_fname=&My_dta.d_fname[4];
		My_dta.d_pfad=&My_dta.d_fname[My_dta.d_max_name];
		My_dta.d_sattr=attr;
		if(Pdomain(-1)==1)
		{	if((My_dta.d_handle=Dopendir(".",0))!=-32l)
			{	My_dta.d_flags=DTF_DO;
				if(Fxattr(0,".",&xdummy)!=-32)
					My_dta.d_flags|=DTF_FX;
				if(Dxreaddir(FILENAME_MAX,My_dta.d_handle,My_dta.d_fname,&xdummy,&ldummy)!=-32)
					My_dta.d_flags|=DTF_FX;
				Dclosedir(My_dta.d_handle);
			}
		}
	}
	else
	{
		if((My_dta.d_flags&DTF_DO)!=0 && My_dta.d_handle>-1)
			Dclosedir(My_dta.d_handle);
	}
	if((My_dta.d_flags&DTF_DO)!=0)
	{	trenn_pfad(My_dta.d_pfad,My_dta.d_maske,filename);
		if(!My_dta.d_pfad[0])
			strcpy(My_dta.d_pfad,".");
		if(case_ign)
			My_dta.d_case_s=case_ign;
		else
		{	if(Dpathconf(My_dta.d_pfad,-1)>5l)
				My_dta.d_case_s=(int)Dpathconf(My_dta.d_pfad,6);
			else
				My_dta.d_case_s=CASE_CHG;
		}
		if(((My_dta.d_handle=Dopendir(My_dta.d_pfad,0))&0xff000000l)==0l)
			rueck=fsnext();
		else
			rueck=(int)My_dta.d_handle;
	}
	else
	{	if((rueck=Fsfirst(filename,attr))==0)
		{	memcpy(&My_dta,&Dta,(int)offsetof(DTAX,d_max_pfad));
			strcpy(My_dta.d_fname,Dta.d_fname);
		}
		My_dta.d_max_pfad=PATH_MAX;
		My_dta.d_max_name=12;
	}
	return(rueck);
}
/*------------------------------------------------------------------*/
/*	int fsnext(void);												*/
/*																	*/
/*	Wie das groe Vorbild, aber auch fr alternative Filesysteme	*/
/*																	*/
/*	Parameter:nur die in My_dta										*/
/*	Rckgabe :Die Funktion liefert als Ergebnis 0, wenn eine Datei	*/
/*              gefunden werden konnte, ansonsten einen Fehlercode.	*/
/*			die eigentliche Rckgabe geschiet in My_dta				*/
/*	(c) by Dimitri Junker 1996										*/
/*------------------------------------------------------------------*/
int fsnext(void)
{	int rueck;
	XATTR xattr;
	char name[PATH_MAX+FILENAME_MAX];
	long xret=-32l;
	if((My_dta.d_flags&DTF_DO)==0)
	{	if((rueck=Fsnext())==0)
		{	memcpy(&My_dta,&Dta,(int)offsetof(DTAX,d_max_pfad));
			strcpy(My_dta.d_fname,Dta.d_fname);
		}
	}
	else
	{	do
		{	if((My_dta.d_flags&DTF_DX)==0)
				rueck=(int)Dreaddir(My_dta.d_max_name,My_dta.d_handle,&My_dta.d_fname[-4]);
			else
				rueck=(int)Dxreaddir(My_dta.d_max_name,My_dta.d_handle,&My_dta.d_fname[-4],&xattr,&xret);
			switch(rueck)
			{	case 0:
					rueck=1-cmp_maske(My_dta.d_fname,My_dta.d_maske,My_dta.d_case_s);
					break;
				case -64:
					My_dta.d_max_name*=2;
					My_dta.d_fname=realloc(&My_dta.d_fname[-4],My_dta.d_max_name+My_dta.d_max_pfad+4);
					My_dta.d_fname=&My_dta.d_fname[4];
					My_dta.d_pfad=&My_dta.d_fname[My_dta.d_max_name];
					rueck=1;
					break;
			}
			if(!rueck)	/* dann noch die Attribute testen	*/
			{	if((My_dta.d_flags&(DTF_DX|DTF_FX))==DTF_FX)
				{	build_filename(name,My_dta.d_pfad,My_dta.d_fname,FALSE);
					xret=Fxattr(0,name,&xattr);
				}
				if(!xret)
				{	if(((xattr.attr!=0)&&(My_dta.d_sattr!=8))||((My_dta.d_sattr|0x21)&xattr.attr))
					{	My_dta.d_attrib=xattr.attr;
						My_dta.d_time=xattr.mtime;
						My_dta.d_date=xattr.mdate;
						My_dta.d_length=xattr.size;
					}
					else
						rueck=1;
				}
			}
		}while(rueck==1);
	}
	return(rueck);
}
/*------------------------------------------------------------------*/
/*	int cmp_maske(char *str,char *maske);							*/
/*																	*/
/*	Prft ob str mit maske vertrglich ist.							*/
/*																	*/
/*	Parameter:	str ein String										*/
/*				maske die maske, mit * und ?						*/
/*			ign_case: wenn <>0 wird Gro-/Kleinschreibung ignoriert */
/*  Achtung *? ist nicht =* sondern irgendwas gefolgt von  '?'		*/
/*	Rckgabe :Die Funktion liefert als Ergebnis 1, wenn 			*/
/*             beide vertrglich sind, sonst 0						*/
/*	(c) by Dimitri Junker 1996										*/
/*------------------------------------------------------------------*/
int cmp_maske(char *str,char *maske,int ign_case)
{	char *s_ptr;
	int gleich=0;
	
	switch((int)*maske)
	{	case '*':
			s_ptr=str;
			while((s_ptr=strchr(s_ptr,(int)maske[1]))!=NULL && !gleich)
				gleich=cmp_maske(s_ptr++,&maske[1],ign_case);
			break;
		case '?':
			if(*str)
				gleich=cmp_maske(&str[1],&maske[1],ign_case);
			break;
		case '\0':
			gleich=(*str=='\0');
			break;
		default:
			if(*str==*maske || (ign_case&& (toupper(*str)==toupper(*maske))))
				gleich=cmp_maske(&str[1],&maske[1],ign_case);
	}
	return(gleich);
}

FILE *tmp_tmpfile(void)
{	char akt_path[PATH_MAX];
	FILE *file;
	get_akt_path(akt_path);
	set_akt_path(Tmpdir);
	file=tmpfile();
	set_akt_path(akt_path);
	return(file);
}
/*** ------------------------------------------------------------------------ ***/
/*						*/
/*								*/
/*	Parameter:	keine															*/
/*	Ergebnis:	keines															*/		
/*	5.6.02 von Dimitri Junker	DYJ 1.05b										*/
/*** ------------------------------------------------------------------------ ***/
void tmpFilename(char *buf,char* ext)
{
	char *ptrTmp,tmp[PATH_MAX];
	ptrTmp=getenv("TMPDIR");
	if(!ptrTmp)
		build_filename(buf,get_akt_path(tmp),tmpnam(NULL),TRUE);
	else
		build_filename(buf,ptrTmp,tmpnam(NULL),TRUE);
	if(*ext)
		strcpy(strrchr(buf,'.')+1,ext);
	
}

/*** ------------------------------------------------------------------------ ***/
/*	void unQuote(char *ziel, const char *quell)									*/
/*	entfernt das Quoting														*/
/*	Parameter:	ziel, quell:na was wohl											*/
/*				blkIsEnd: Ist ein Blk auerhalb des Quotings das Ende des Parameters?*/
/*	Ergebnis:	keines															*/		
/*	8.11.00 von Dimitri Junker													*/
/*** ------------------------------------------------------------------------ ***/
void unQuote(char *ziel, const char *quell,BOOLEAN blkIsEnd)
{
	while(*quell==' ')
		quell++;
	if(*quell=='\'')
	{
		while(*quell && (quell[0]!='\'' || quell[1]=='\''))
		{
			*(ziel++)=*(quell++);
			if(*quell=='\'')
				quell++;
		}
		*ziel='\0';
	}
	else if(!blkIsEnd)
		strcpy(ziel,quell);
	else
	{
		while(*quell && *quell!=' ')
			*(ziel++)=*(quell++);
		*ziel='\0';
	}
}

/*----------------------- Ende der Datei. --------------------------*/
	