/*			I2C_LOG.C V 2.0
		(c) 2003 by Dimitri Junker
			Adenauerallee 30
			52066 Aachen

		Dieses File ist Teil des In2Cat-Projektes und steht unter der
		In2Cat Public License. Dementspechend ist es Freeware und darf
		entsprechend der o.g. Liscense weitergegeben und verndert werden.
		Eine Kopie der In2Cat Public License liegt diesem Paket bei
		Diese Copyright Meldung darf nur vom Copyright Inhaber gendert werden.
*/

/* -------------------------------------------------------------------- */
/*		Includes														*/
/* -------------------------------------------------------------------- */

#include <easyfsel.h>
#include <I2C_LOG.h>
#include <time.h>
#include <string.h>

/* -------------------------------------------------------------------- */
/*		Globale Variablen												*/
/* -------------------------------------------------------------------- */
FILE *Logfile=NULL;
extern char 	LogLevel=0;

/* -------------------------------------------------------------------- */
/*		Funktionen														*/
/* -------------------------------------------------------------------- */

/*** ------------------------------------------------------------------------ ***/
/*	int logPrintf(int level,BOOLEAN nl,const char *format1,... )				*/
/*	Gibt eine Zeile ins LOG aus. Mit Zeitangabe,...								*/
/*	Parameter:	level: Priorittslevel (0=Gering, 2=Hoch						*/
/*				nl:			zuerst ein Newline?									*/
/*				format1:	Formatstring										*/
/*				...:		andere Parameter									*/
/*	Ergebnis:	wie bei printf													*/		
/*	20.10.02 von Dimitri Junker	DYJ 1.22b										*/
/*** ------------------------------------------------------------------------ ***/
int logPrintf(int level,BOOLEAN nl,const char *format1,... )
{
	char format2[200]="\n%ld: ";
	int msg=EOF;
	va_list pvar;
	
	if(Logfile && level>=LogLevel)
	{
		if(nl)
			sprintf(format2,"\n%ld: %s",clock(),format1);
		else
			sprintf(format2,"%ld: %s",clock(),format1);
		va_start(pvar,format1);
		msg=vfprintf(Logfile,format2,pvar);
		va_end(pvar);
		fflush(Logfile);
	}
	return msg;
}

void logErr(char *txt,int nr)
{
	const char *msg=NULL;
	char tmsg[100],pref[10]="???";
	switch(nr&0xf000)
	{
		case ERR_RAS:
			strcpy(pref,"RAS");
			switch(nr&0xfff)
			{
				case 5:		msg="Invalid procedure call.";break;
				case 6:		msg="Overflow.";break;
				case 7:		msg="Out of memory.";break;
				case 9:		msg="Subscript out of range.";break;
				case 10:	msg="This array is fixed or temporarily locked.";break;
				case 11:	msg="Division by zero.";break;
				case 13:	msg="Type mismatch.";break;
				case 16:	msg="Expression too complex.";break;
				case 17:	msg="Can't perform requested operation.";break;
				case 18:	msg="User interrupt occurred.";break;
				case 20:	msg="Resume without error.";break;
				case 28:	msg="Out of stack space.";break;
				case 35:	msg="Sub or Function not defined.";break;
				case 47:	msg="Too many DLL application clients.";break;
				case 48:	msg="Error in loading DLL.";break;
				case 49:	msg="Bad DLL calling convention.";break;
				case 51:	msg="Internal error.";break;
				case 52:	msg="Bad file name or number.";break;
				case 53:	msg="File not found.";break;
				case 54:	msg="Bad file mode.";break;
				case 55:	msg="File already open.";break;
				case 57:	msg="Device I/O error.";break;
				case 58:	msg="File already exists.";break;
				case 59:	msg="Bad record length.";break;
				case 61:	msg="Disk full.";break;
				case 62:	msg="Input past end of file.";break;
				case 63:	msg="Bad record number.";break;
				case 67:	msg="Too many files.";break;
				case 68:	msg="Device unavailable.";break;
				case 70:	msg="Permission denied.";break;
				case 71:	msg="Disk not ready.";break;
				case 74:	msg="Can't rename with different drive.";break;
				case 75:	msg="Path/File access error.";break;
				case 76:	msg="Path not found.";break;
				case 81:	msg="/TEXT/ is read only.";break;
				case 91:	msg="Object variable or With block variable not set.";break;
				case 92:	msg="For loop not initialized.";break;
				case 93:	msg="Invalid pattern string.";break;
				case 94:	msg="Invalid use of Null.";break;
				case 322:	msg="Can't create necessary temporary file.";break;
				case 325:	msg="Invalid format in resource file.";break;
				case 367:	msg="Cannot load or register custom control Esras32.ocx or sstabs32.ocx.";break;
				case 380:	msg="Invalid property value.";break;
				case 383:	msg="Text property is read only.";break;
				case 423:	msg="Property or method not found.";break;
				case 424:	msg="Object required.";break;
				case 429:	msg="OLE Automation server can't create object.";break;
				case 430:	msg="Class doesn't support OLE Automation.";break;
				case 432:	msg="File name or class name not found during OLE Automation operation.";break;
				case 438:	msg="Object doesn't support this property or method.";break;
				case 440:	msg="OLE Automation error.";break;
				case 442:	msg="Connection to type library or object library for remote process has been lost. Press OK for dialog to remove reference.";break;
				case 443:	msg="OLE Automation object does not have a default value.";break;
				case 445:	msg="Object doesn't support this action.";break;
				case 446:	msg="Object doesn't support named arguments.";break;
				case 447:	msg="Object doesn't support current locale setting.";break;
				case 448:	msg="Named argument not found.";break;
				case 449:	msg="Argument not optional.";break;
				case 450:	msg="Wrong number of arguments or invalid property assignment.";break;
				case 451:	msg="Object not a collection.";break;
				case 452:	msg="Invalid ordinal.";break;
				case 453:	msg="Specified DLL function not found.";break;
				case 454:	msg="Code resource not found.";break;
				case 455:	msg="Code resource lock error.";break;
				case 457:	msg="This key is already associated with an element of this collection.";break;
				case 458:	msg="Variable uses an OLE type not supported in Visual Basic.";break;
				case 481:	msg="Invalid picture.";break;
				case 502:	msg="You have no permission to talk.";break;
				case 600:	msg="An operation is pending.";break;
				case 601:	msg="The port handle is invalid.";break;
				case 602:	msg="The port is already open.";break;
				case 603:	msg="Caller's buffer is too small.";break;
				case 604:	msg="Wrong information specified.";break;
				case 605:	msg="Cannot set port information.";break;
				case 606:	msg="The port is not connected.";break;
				case 607:	msg="The event is invalid.";break;
				case 608:	msg="The device does not exist.";break;
				case 609:	msg="The device type does not exist.";break;
				case 610:	msg="The buffer is invalid.";break;
				case 611:	msg="The route is not available.";break;
				case 612:	msg="The route is not allocated.";break;
				case 613:	msg="Invalid compression specified.";break;
				case 614:	msg="Out of buffers.";break;
				case 615:	msg="The port was not found.";break;
				case 616:	msg="An asynchronous request is pending.";break;
				case 617:	msg="The port or device is already disconnecting.";break;
				case 618:	msg="The port is not open.";break;
				case 619:	msg="The port is disconnected.";break;
				case 620:	msg="There are no endpoints.";break;
				case 621:	msg="Cannot open the phone book file.";break;
				case 622:	msg="Cannot load the phone book file.";break;
				case 623:	msg="Cannot find the phone book entry.";break;
				case 624:	msg="Cannot write the phone book file.";break;
				case 625:	msg="Invalid information found in the phone book.";break;
				case 626:	msg="Cannot load a string.";break;
				case 627:	msg="Cannot find key.";break;
				case 628:	msg="The port was disconnected.";break;
				case 629:	msg="The port was disconnected by the remote machine.";break;
				case 630:	msg="The port was disconnected due to hardware failure.";break;
				case 631:	msg="The port was disconnected by the user.";break;
				case 632:	msg="The structure size is incorrect.";break;
				case 633:	msg="The port is already in use or is not configured for Remote Access dialout.";break;
				case 634:	msg="Cannot register your computer on the remote network.";break;
				case 635:	msg="Unknown error.";break;
				case 636:	msg="The wrong device is attached to the port.";break;
				case 637:	msg="The string could not be converted.";break;
				case 638:	msg="The request has timed out.";break;
				case 639:	msg="No asynchronous net available.";break;
				case 640:	msg="A NetBIOS error has occurred.";break;
				case 641:	msg="The server cannot allocate NetBIOS resources needed to support the client.";break;
				case 642:	msg="One of your NetBIOS names is already registered on the remote network.";break;
				case 643:	msg="A network adapter at the server failed.";break;
				case 644:	msg="You will not receive network message popups.";break;
				case 645:	msg="Internal authentication error.";break;
				case 646:	msg="The account is not permitted to log on at this time of day.";break;
				case 647:	msg="The account is disabled.";break;
				case 648:	msg="The password has expired.";break;
				case 649:	msg="The account does not have Remote Access permission.";break;
				case 650:	msg="The Remote Access server is not responding.";break;
				case 651:	msg="Your modem (or other connecting device) has reported an error.";break;
				case 652:	msg="Unrecognized response from the device.";break;
				case 653:	msg="A macro required by the device was not found in the device .INF file section.";break;
				case 654:	msg="A command or response in the device .INF file section refers to an undefined macro.";break;
				case 655:	msg="The macro was not found in the device .INF file section.";break;
				case 656:	msg="The macro in the device .INF file section contains an undefined macro.";break;
				case 657:	msg="The device .INF file could not be opened.";break;
				case 658:	msg="The device name in the device .INF or media .INI file is too long.";break;
				case 659:	msg="The media .INI file refers to an unknown device name.";break;
				case 660:	msg="The device .INF file contains no responses for the command.";break;
				case 661:	msg="The device .INF file is missing a command.";break;
				case 662:	msg="Attempted to set a macro not listed in device .INF file section.";break;
				case 663:	msg="The media .INI file refers to an unknown device type.";break;
				case 664:	msg="Cannot allocate memory.";break;
				case 665:	msg="The port is not configured for Remote Access.";break;
				case 666:	msg="Your modem (or other connecting device) is not functioning.";break;
				case 667:	msg="Cannot read the media .INI file.";break;
				case 668:	msg="The connection dropped.";break;
				case 669:	msg="The usage parameter in the media .INI file is invalid.";break;
				case 670:	msg="Cannot read the section name from the media .INI file.";break;
				case 671:	msg="Cannot read the device type from the media .INI file.";break;
				case 672:	msg="Cannot read the device name from the media .INI file.";break;
				case 673:	msg="Cannot read the usage from the media .INI file.";break;
				case 674:	msg="Cannot read the maximum connection BPS rate from the media .INI file.";break;
				case 675:	msg="Cannot read the maximum carrier BPS rate from the media .INI file.";break;
				case 676:	msg="The line is busy.";break;
				case 677:	msg="A person answered instead of a modem.";break;
				case 678:	msg="There is no answer.";break;
				case 679:	msg="Cannot detect carrier.";break;
				case 680:	msg="There is no dial tone.";break;
				case 681:	msg="General error reported by device.";break;
				case 682:	msg="ERROR WRITING SECTIONNAME";break;
				case 683:	msg="ERROR WRITING DEVICETYPE";break;
				case 684:	msg="ERROR WRITING DEVICENAME";break;
				case 685:	msg="ERROR WRITING MAXCONNECTBPS";break;
				case 686:	msg="ERROR WRITING MAXCARRIERBPS";break;
				case 687:	msg="ERROR WRITING USAGE";break;
				case 688:	msg="ERROR WRITING DEFAULTOFF";break;
				case 689:	msg="ERROR READING DEFAULTOFF";break;
				case 690:	msg="ERROR EMPTY INI FILE";break;
				case 691:	msg="Access denied because username and/or password is invalid on the domain.";break;
				case 692:	msg="Hardware failure in port or attached device.";break;
				case 693:	msg="ERROR NOT BINARY MACRO";break;
				case 694:	msg="ERROR DCB NOT FOUND";break;
				case 695:	msg="ERROR STATE MACHINES NOT STARTED";break;
				case 696:	msg="ERROR STATE MACHINES ALREADY STARTED";break;
				case 697:	msg="ERROR PARTIAL RESPONSE LOOPING";break;
				case 698:	msg="A response keyname in the device .INF file is not in the expected format.";break;
				case 699:	msg="The device response caused buffer overflow.";break;
				case 700:	msg="The expanded command in the device .INF file is too long.";break;
				case 701:	msg="The device moved to a BPS rate not supported by the COM driver.";break;
				case 702:	msg="Device response received when none expected.";break;
				case 703:	msg="ERROR INTERACTIVE MODE";break;
				case 704:	msg="ERROR BAD CALLBACK NUMBER";break;
				case 705:	msg="ERROR INVALID AUTH STATE";break;
				case 706:	msg="ERROR WRITING INITBPS";break;
				case 707:	msg="X.25 diagnostic indication.";break;
				case 708:	msg="The account has expired.";break;
				case 709:	msg="Error changing password on domain.";break;
				case 710:	msg="Serial overrun errors were detected while communicating with your modem.";break;
				case 711:	msg="RasMan initialization failure. Check the event log.";break;
				case 712:	msg="Biplex port is initializing. Wait a few seconds and redial.";break;
				case 713:	msg="No active ISDN lines are available.";break;
				case 714:	msg="Not enough ISDN channels are available to make the call.";break;
				case 715:	msg="Too many errors occurred because of poor phone line quality.";break;
				case 716:	msg="The Remote Access IP configuration is unusable.";break;
				case 717:	msg="No IP addresses are available in the static pool of Remote Access IP addresses.";break;
				case 718:	msg="PPP timeout.";break;
				case 719:	msg="PPP terminated by remote machine.";break;
				case 720:	msg="No PPP control protocols configured.";break;
				case 721:	msg="Remote PPP peer is not responding.";break;
				case 722:	msg="The PPP packet is invalid.";break;
				case 723:	msg="The phone number, including prefix and suffix, is too long.";break;
				case 724:	msg="The IPX protocol cannot dial-out on the port because the computer is an IPX router.";break;
				case 725:	msg="The IPX protocol cannot dial-in on the port because the IPX router is not installed.";break;
				case 726:	msg="The IPX protocol cannot be used for dial-out on more than one port at a time.";break;
				case 727:	msg="Cannot access TCPCFG.DLL.";break;
				case 728:	msg="Cannot find an IP adapter bound to Remote Access.";break;
				case 729:	msg="SLIP cannot be used unless the IP protocol is installed.";break;
				case 730:	msg="Computer registration is not complete.";break;
				case 731:	msg="The protocol is not configured.";break;
				case 732:	msg="The PPP negotiation is not converging.";break;
				case 733:	msg="The PPP control protocol for this network protocol is not available on the server.";break;
				case 734:	msg="The PPP link control protocol terminated.";break;
				case 735:	msg="The requested address was rejected by the server.";break;
				case 736:	msg="The remote computer terminated the control protocol.";break;
				case 737:	msg="Loopback detected.";break;
				case 738:	msg="The server did not assign an address.";break;
				case 739:	msg="The remote server cannot use the Windows NT encrypted password.";break;
				case 740:	msg="The TAPI devices configured for Remote Access failed to initialize or were not installed correctly.";break;
				case 741:	msg="The local computer does not support encryption.";break;
				case 742:	msg="The remote server does not support encryption.";break;
				case 743:	msg="The remote server requires encryption.";break;
				case 744:	msg="Cannot use the IPX net number assigned by the remote server. Check the event log.";break;
				case 745:	msg="ERROR_INVALID_SMM%0";break;
				case 746:	msg="ERROR_SMM_UNINITIALIZED%0";break;
				case 747:	msg="ERROR_NO_MAC_FOR_PORT%0";break;
				case 748:	msg="ERROR_SMM_TIMEOUT%0";break;
				case 749:	msg="ERROR_BAD_PHONE_NUMBER%0";break;
				case 750:	msg="ERROR_WRONG_MODULE%0";break;
				case 751:	msg="The callback number contains an invalid character. Only the following characters are allowed: 0 to 9, T, P, W, (,), -, @, and space";break;
				case 752:	msg="A syntax error was encountered while processing a script.";break;
				case 753:	msg="The connection could not be disconnected because it was created by the multi-protocol router";break;
				case 754:	msg="The system could not find the multi-link bundle";break;
				case 755:	msg="The system cannot perform automated dial because this entry has a custom dialer specified";break;
				case 756:	msg="This connection is already being dialed";break;
				case 757:	msg="Remote access services could not be started automatically. Additional information is provided in the event log";break;
				case 758:	msg="Internet Connection Sharing is already enabled on the connection";break;
				case 760:	msg="An error occurred while routing capabilities were being enabled";break;
				case 761:	msg="An error occurred while Internet Connection Sharing was being enabled for the connection";break;
				case 763:	msg="Internet Connection Sharing cannot be enabled. There are two or more LAN connections in addition to the connection to be shared";break;
				case 764:	msg="No smart card reader is installed";break;
				case 765:	msg="Internet Connection Sharing cannot be enabled. A LAN connection is already configured with the IP address required for automatic IP addressing";break;
				case 766:	msg="The system could not find any certificate";break;
				case 767:	msg="Internet Connection Sharing cannot be enabled. The LAN connection selected on the private network has more than one IP address configured. Reconfigure the LAN connection with a single IP address before enabling \
Internet Connection Sharing";break;
				case 768:	msg="The connection attempt failed because of failure to encrypt data";break;
				case 769:	msg="The specified destination is not reachable";break;
				case 770:	msg="The remote machine rejected the connection attempt";break;
				case 771:	msg="The connection attempt failed because the network is busy";break;
				case 772:	msg="The remote computer's network hardware is incompatible with the type of call requested";break;
				case 773:	msg="The connection attempt failed because the destination number has changed";break;
				case 774:	msg="The connection attempt failed because of a temporary failure. Try connecting again";break;
				case 775:	msg="The call was blocked by the remote computer";break;
				case 776:	msg="The call could not be connected because the destination has invoked the Do Not Disturb feature";break;
				case 777:	msg="The connection attempt failed because the modem on the remote computer is out of order";break;
				case 778:	msg="It was not possible to verify the identity of the server";break;
				case 779:	msg="To dial out using this connection, you must use a smart card";break;
				case 780:	msg="An attempted function is not valid for this connection";break;
				case 781:	msg="The encryption attempt failed because no valid certificate was found";break;
				case 782:	msg="Network Address Translation (NAT) is currently installed as a routing protocol, and must be removed before enabling Internet Connection Sharing";break;
				case 783:	msg="Internet Connection Sharing cannot be enabled. The LAN connection selected as the private network is either not present, or is disconnected from the network. Please ensure that the LAN adapter is connected before \
enabling Internet Connection Sharing";break;
				case 784:	msg="You cannot dial using this connection at logon time, because it is configured to use a user name different than the one on the smart card. If you want to use it at logon time, you must configure it to use the user \
name on the smart card";break;
				case 785:	msg="You cannot dial using this connection at logon time, because it is not configured to use a smart card. If you want to use it at logon time, you must edit the properties of this connection so that it uses a smart \
card";break;
				case 786:	msg="The L2TP connection attempt failed because there is no valid machine certificate on your computer for security authentication";break;
				case 787:	msg="The L2TP connection attempt failed because the security layer could not authenticate the remote computer";break;
				case 788:	msg="The L2TP connection attempt failed because the security layer could not negotiate compatible parameters with the remote computer";break;
				case 789:	msg="The L2TP connection attempt failed because the security layer encountered a processing error during initial negotiations with the remote computer";break;
				case 790:	msg="The L2TP connection attempt failed because certificate validation on the remote computer failed";break;
				case 791:	msg="The L2TP connection attempt failed because security policy for the connection was not found";break;
				case 792:	msg="The L2TP connection attempt failed because security negotiation timed out";break;
				case 793:	msg="The L2TP connection attempt failed because an error occurred while negotiating security";break;
				case 794:	msg="The Framed Protocol RADIUS attribute for this user is not PPP";break;
				case 795:	msg="The Tunnel Type RADIUS attribute for this user is not correct";break;
				case 796:	msg="The Service Type RADIUS attribute for this user is neither Framed nor Callback Framed";break;
				case 797:	msg="A connection to the remote computer could not be established because the modem was not found or was busy";break;
				case 798:	msg="A certificate could not be found that can be used with this Extensible Authentication Protocol";break;
				case 799:	msg="Internet Connection Sharing (ICS) cannot be enabled due to an IP address conflict on the network. ICS requires the host be configured to use 192.168.0.1. Please ensure that no other client on the network is \
configured to use 192.168.0.1";break;
				case 800:	msg="Unable to establish the VPN connection. The VPN server may be un-reachable, or security parameters may not be configured properly for this connection";break;
			}
			break;
		case ERR_ST:
			strcpy(pref,"ST");
			switch(nr&0xfff)
			{
				case 1:	msg="Output buffer is full";break;
				case 2:	msg="No data available";break;
				case 3:	msg="EOF from remote";break;
				case 4:	msg="Reset received from remote";break;
				case 5:	msg="Unacceptable packet received, reset";break;
				case 6:	msg="Something failed due to lack of memory";break;
				case 7:	msg="Connection refused by remote";break;
				case 8:	msg="A SYN was received in the window";break;
				case 9:	msg="Bad connection handle used.";break;
				case 10:	msg="The connection is in LISTEN state";break;
				case 11:	msg="No free CCB's available";break;
				case 12:	msg="No connection matches this packet (TCP)";break;
				case 13:	msg="Failure to connect to remote port (TCP)";break;
				case 14:	msg="Invalid TCP_close() requested";break;
				case 15:	msg="A user function timed out";break;
				case 16:	msg="A connection timed out";break;
				case 17:	msg="Can't resolve the hostname";break;
				case 18:	msg="Domain name or dotted dec. bad format";break;
				case 19:	msg="The modem disconnected";break;
				case 20:	msg="Hostname does not exist";break;
				case 21:	msg="Resolver Work limit reached";break;
				case 22:	msg="No nameservers could be found for query";break;
				case 23:	msg="Bad format of DNS query";break;
				case 24:	msg="Destination unreachable";break;
				case 25:	msg="No address records exist for host";break;
				case 26:	msg="Routine unavailable";break;
				case 27:	msg="Locked by another application";break;
				case 28:	msg="Error during fragmentation";break;
				case 29:	msg="Time To Live of an IP packet exceeded";break;
				case 30:	msg="Problem with a parameter";break;
				case 31:	msg="Input buffer is too small for data";break;
				case 32:	msg="Function not available";break;
			}
			break;
		case ERR_IC:
			strcpy(pref,"IC");
			switch(nr&0xfff)
			{
				case 300:	msg="Socket operation on non-socket";break;
				case 301:	msg="Destination address required";break;
				case 302:	msg="Message too long";break;
				case 303:	msg="Protocol wrong type for socket";break;
				case 304:	msg="Protocol not available";break;
				case 305:	msg="Protocol not supported";break;
				case 306:	msg="Socket type not supported";break;
				case 307:	msg="Operation not supported";break;
				case 308:	msg="Protocol family not supported";break;
				case 309:	msg="Address family not supported by protocol";break;
				case 310:	msg="Address already in use";break;
				case 311:	msg="Cannot assign requested address";break;
				case 312:	msg="Network is down";break;
				case 313:	msg="Network is unreachable";break;
				case 314:	msg="Network dropped conn. because of reset";break;
				case 315:	msg="Software caused connection abort";break;
				case 316:	msg="Connection reset by peer";break;
				case 317:	msg="Socket is already connected";break;
				case 318:	msg="Socket is not connected";break;
				case 319:	msg="Cannot send after shutdown";break;
				case 320:	msg="Connection timed out";break;
				case 321:	msg="Connection refused";break;
				case 322:	msg="Host is down";break;
				case 323:	msg="No route to host";break;
				case 324:	msg="Operation already in progress";break;
				case 325:	msg="Operation now in progress";break;
				case 326:	msg="Operation would block";break;
				case 128:	msg="System call was interrupted";break;
			}
			break;
		case ERR_10K:
			strcpy(pref,"");
			switch(nr)
			{
				case 10013:	msg="Permission denied. An attempt was made to access a socket in a way forbidden by its access permissions. An example is using a broadcast address for sendto without broadcast permission being set using setsockopt\
				(SO_BROADCAST).";break;

				case 10048:	msg="Address already in use. Only one usage of each socket address (protocol/IP address/port) is normally permitted. This error occurs if an application attempts to bind a socket to an IP address/port that has already \
been used for an existing socket, or a socket that wasn't closed properly, or one that is still in the process of closing. For server applications that need to bind multiple sockets to the same port number, consider using \
setsockopt(SO_REUSEADDR). Client applications usually need not call bind at all - connect will choose an unused port automatically. When bind is called with a wild-card address (involving ADDR_ANY), a WSAEADDRINUSE error could be \
delayed until the specific address is \"committed.\" This could happen with a call to other function later, including connect, listen, WSAConnect or WSAJoinLeaf.";break;

				case 10049:	msg="Cannot assign requested address. The requested address is not valid in its context. Normally results from an attempt to bind to an address that is not valid for the local machine. This can also result from connect, \
sendto, WSAConnect, WSAJoinLeaf, or WSASendTo when the remote address or port is not valid for a remote machine (e.g. address or port 0).";break;

				case 10047:	msg="Address family not supported by protocol family. An address incompatible with the requested protocol was used. All sockets are created with an associated \"address family\" (i.e. AF_INET for Internet Protocols) and a \
generic protocol type (i.e. SOCK_STREAM). This error will be returned if an incorrect protocol is explicitly requested in the socket call, or if an address of the wrong family is used for a socket, e.g. in sendto.";break;

				case 10037:	msg="Operation already in progress. An operation was attempted on a non-blocking socket that already had an operation in progress - i.e. calling connect a second time on a non-blocking socket that is already connecting, or \
canceling an asynchronous request (WSAAsyncGetXbyY) that has already been canceled or completed.";break;

				case 10053:	msg="Software caused connection abort. An established connection was aborted by the software in your host machine, possibly due to a data transmission timeout or protocol error.";break;

				case 10061:	msg="Connection refused. No connection could be made because the target machine actively refused it. This usually results from trying to connect to a service that is inactive on the foreign host - i.e. one with no server \
application running.";break;
				case 10054:	msg="Connection reset by peer. A existing connection was forcibly closed by the remote host. This normally results if the peer application on the remote host is suddenly stopped, the host is rebooted, or the remote host used \
a \"hard close\" (see setsockopt for more information on the SO_LINGER option on the remote socket.) This error may also result if a connection was broken due to \"keep-alive\" activity detecting a failure while one or more operations \
are in progress. Operations that were in progress fail with WSAENETRESET. Subsequent operations fail with WSAECONNRESET.";break;
				case 10039:	msg="Destination address required. A required address was omitted from an operation on a socket. For example, this error will be returned if sendto is called with the remote address of ADDR_ANY.";break;
				case 10014:	msg="Bad address. The system detected an invalid pointer address in attempting to use a pointer argument of a call. This error occurs if an application passes an invalid pointer value, or if the length of the buffer is too \
small. For instance, if the length of an argument which is a struct sockaddr is smaller than sizeof(struct sockaddr).";break;
				case 10064:	msg="Host is down. A socket operation failed because the destination host was down. A socket operation encountered a dead host. Networking activity on the local host has not been initiated. These conditions are more likely to \
be indicated by the error WSAETIMEDOUT.";break;
				case 10065:	msg="No route to host. A socket operation was attempted to an unreachable host. See WSAENETUNREACH";break;
				case 10036:	msg="Operation now in progress. A blocking operation is currently executing. Windows Sockets only allows a single blocking operation to be outstanding per task (or thread), and if any other function call is made (whether or \
not it references that or any other socket) the function fails with the WSAEINPROGRESS error.";break;
				case 10004:	msg="Interrupted function call. A blocking operation was interrupted by a call to WSACancelBlockingCall.";break;
				case 10022:	msg="Invalid argument. Some invalid argument was supplied (for example, specifying an invalid level to the setsockopt function). In some instances, it also refers to the current state of the socket - for instance, calling \
accept on a socket that is not listening.";break;
				case 10056:	msg="Socket is already connected. A connect request was made on an already connected socket. Some implementations also return this error if sendto is called on a connected SOCK_DGRAM socket (For SOCK_STREAM sockets, the to \
parameter in sendto is ignored), although other implementations treat this as a legal occurrence.";break;
				case 10024:	msg="Too many open files. Too many open sockets. Each implementation may have a maximum number of socket handles available, either globally, per process or per thread.";break;
				case 10040:	msg="Message too long. A message sent on a datagram socket was larger than the internal message buffer or some other network limit, or the buffer used to receive a datagram into was smaller than the datagram itself.";break;
				case 10050:	msg="Network is down. A socket operation encountered a dead network. This could indicate a serious failure of the network system (i.e. the protocol stack that the WinSock DLL runs over), the network interface, or the local \
network itself.";break;
				case 10052:	msg="Network dropped connection on reset. The connection has been broken due to \"keep-alive\" activity detecting a failure while the operation was in progress. It can also be returned by setsockopt if an attempt is made to \
set SO_KEEPALIVE on a connection that has already failed.";break;
				case 10051:	msg="Network is unreachable. A socket operation was attempted to an unreachable network. This usually means the local software knows no route to reach the remote host.";break;
				case 10055:	msg="No buffer space available. An operation on a socket could not be performed because the system lacked sufficient buffer space or because a queue was full.";break;
				case 10042:	msg="Bad protocol option. An unknown, invalid or unsupported option or level was specified in a getsockopt or setsockopt call.";break;
				case 10057:	msg="Socket is not connected. A request to send or receive data was disallowed because the socket is not connected and (when sending on a datagram socket using sendto) no address was supplied. Any other type of operation \
might also return this error - for example, setsockopt setting SO_KEEPALIVE if the connection has been reset.";break;
				case 10038:	msg="Socket operation on non-socket. An operation was attempted on something that is not a socket. Either the socket handle parameter did not reference a valid socket, or for select, a member of an fd_set was not valid.";break;
				case 10045:	msg="Operation not supported. The attempted operation is not supported for the type of object referenced. Usually this occurs when a socket descriptor to a socket that cannot support this operation, for example, trying to \
accept a connection on a datagram socket.";break;
				case 10046:	msg="Protocol family not supported. The protocol family has not been configured into the system or no implementation for it exists. Has a slightly different meaning to WSAEAFNOSUPPORT, but is interchangeable in most cases, \
and all Windows Sockets functions that return one of these specify WSAEAFNOSUPPORT.";break;
				case 10067:	msg="Too many processes. A Windows Sockets implementation may have a limit on the number of applications that may use it simultaneously. WSAStartup may fail with this error if the limit has been reached.";break;
				case 10043:	msg="Protocol not supported. The requested protocol has not been configured into the system, or no implementation for it exists. For example, a socket call requests a SOCK_DGRAM socket, but specifies a stream protocol.";break;
				case 10041:	msg="Protocol wrong type for socket. A protocol was specified in the socket function call that does not support the semantics of the socket type requested. For example, the ARPA Internet UDP protocol cannot be specified with \
a socket type of SOCK_STREAM.";break;
				case 10058:	msg="Cannot send after socket shutdown. A request to send or receive data was disallowed because the socket had already been shut down in that direction with a previous shutdown call. By calling shutdown a partial close of \
a socket is requested, which is a signal that sending or receiving or both has been discontinued.";break;
				case 10044:	msg="Socket type not supported. The support for the specified socket type does not exist in this address family. For example, the optional type SOCK_RAW might be selected in a socket call, and the implementation does not \
support SOCK_RAW sockets at all.";break;
				case 10060:	msg="Connection timed out. A connection attempt failed because the connected party did not properly respond after a period of time, or established connection failed because connected host has failed to respond. \
WSATYPE_NOT_FOUND";break;
				case 10109:	msg="Class type not found. The specified class was not found.";break;
				case 10035:	msg="Resource temporarily unavailable. This error is returned from operations on non-blocking sockets that cannot be completed immediately, for example recv when no data is queued to be read from the socket. It is a \
non-fatal error, and the operation should be retried later. It is normal for WSAEWOULDBLOCK to be reported as the result from calling connect on a non-blocking SOCK_STREAM socket, since some time must elapse for the connection to \
be established. WSAHOST_NOT_FOUND";break;
				case 11001:	msg="Host not found. No such host is known. The name is not an official hostname or alias, or it cannot be found in the database(s) being queried. This error may also be returned for protocol and service queries, and \
means the specified name could not be found in the relevant database. WSA_INVALID_HANDLE";break;
				case 10093:	msg="Successful WSAStartup not yet performed. Either the application hasn't called WSAStartup or WSAStartup failed. The application may be accessing a socket which the current active task does not own (i.e. trying to \
share a socket between tasks), or WSACleanup has been called too many times. WSANO_DATA";break;
				case 11004:	msg="Valid name, no data record of requested type. The requested name is valid and was found in the database, but it does not have the correct associated data being resolved for. The usual example for this is a \
hostname -> address translation attempt (using gethostbyname or WSAAsyncGetHostByName) which uses the DNS (Domain Name Server), and an MX record is returned but no A record - indicating the host itself exists, but is not directly \
reachable. WSANO_RECOVERY";break;
				case 11003:	msg="This is a non-recoverable error. This indicates some sort of non-recoverable error occurred during a database lookup. This may be because the database files (e.g. BSD-compatible HOSTS, SERVICES or PROTOCOLS files) could not be found, or a DNS request was returned by the server with a severe error. WSAPROVIDERFAILEDINIT";break;
				case 10091:	msg="Network subsystem is unavailable. This error is returned by WSAStartup if the Windows Sockets implementation cannot function at this time because the underlying system it uses to provide network services is currently unavailable. Users should check: that the appropriate Windows Sockets DLL file is in the current path, that they are not trying to use more than one Windows Sockets implementation simultaneously. If there is more than one WINSOCK DLL on your system, be sure the first one in the path is appropriate for the network subsystem currently loaded. the Windows Sockets implementation documentation to be sure all necessary components are currently installed and configured correctly. WSATRY_AGAIN";break;
				case 11002:	msg="Non-authoritative host not found. This is usually a temporary error during hostname resolution and means that the local server did not receive a response from an authoritative server. A retry at some time later may be successful. WSAVERNOTSUPPORTED";break;
				case 10092:	msg="WINSOCK.DLL version out of range. The current Windows Sockets implementation does not support the Windows Sockets specification version requested by the application. Check that no old Windows Sockets DLL files are being accessed.";break;
				case 10094:	msg="Graceful shutdown in progress. Returned by WSARecv and WSARecvFrom to indicate the remote party has initiated a graceful shutdown sequence. ";break;
			}
			break;
		case ERR_NNTP:
			strcpy(pref,"NNTP");
			switch(nr&0xfff)
			{
				case 200:	msg="server ready - posting allowed";break;
				case 201:	msg="server ready - no posting allowed";break;
				case 202:	msg="slave status noted";break;
				case 205:	msg="closing connection - goodbye!";break;
				case 211:	msg="n f l s group selected";break;
				case 215:	msg="list of newsgroups follows";break;
				case 220:	msg="n <a> article retrieved - head and body follow ";break;
				case 221:	msg="n <a> article retrieved - head follows";break;
				case 222:	msg="n <a> article retrieved - body follows";break;
				case 223:	msg="n <a> article retrieved - request text separately ";break;
				case 230:	msg="list of new articles by message-id follows";break;
				case 231:	msg="list of new newsgroups follows";break;
				case 235:	msg="article transferred ok";break;
				case 240:	msg="article posted ok";break;
				case 250:	msg="Authorization accepted";break;
				case 281:	msg="Authentication succeeded";break;
				case 288:	msg="Binary data to follow";break;
				case 335:	msg="send article to be transferred.  End with <CR-LF>.<CR-LF>";break;
				case 340:	msg="send article to be posted. End with <CR-LF>.<CR-LF>";break;
				case 350:	msg="Continue with authorization sequence";break;
				case 400:	msg="service discontinued";break;
				case 411:	msg="no such news group";break;
				case 412:	msg="no newsgroup has been selected";break;
				case 420:	msg="no current article has been selected";break;
				case 421:	msg="no next article in this group";break;
				case 422:	msg="no previous article in this group";break;
				case 423:	msg="no such article number in this group";break;
				case 430:	msg="no such article found";break;
				case 435:	msg="article not wanted - do not send it";break;
				case 436:	msg="transfer failed - try again later";break;
				case 437:	msg="article rejected - do not try again.";break;
				case 440:	msg="posting not allowed";break;
				case 441:	msg="posting failed";break;
				case 450:	msg="Authorization required for this command";break;
				case 452:	msg="Authorization rejected";break;
				case 480:	msg="Authentication required";break;
				case 500:	msg="command not recognized";break;
				case 501:	msg="command syntax error";break;
				case 502:	msg="access restriction or permission denied";break;
				case 503:	msg="program fault - command not performed";break;
   			}
			break;
		case ERR_SMTP:
			strcpy(pref,"SMTP");
			switch(nr&0xfff)
			{
				case 421:	msg="<domain> Service not available, closing transmission channel (This may be a reply to any command if the service knows it must shut down)";break;
				case 450:	msg="Requested mail action not taken: mailbox unavailable (e.g., mailbox busy)";break;
				case 451:	msg="Requested action aborted: local error in processing";break;
				case 452:	msg="Requested action not taken: insufficient system storage";break;
				case 500:	msg="Syntax error, command unrecognized (This may include errors such as command line too long)";break;
				case 501:	msg="Syntax error in parameters or arguments";break;
				case 502:	msg="Command not implemented (see section 4.2.4)";break;
				case 503:	msg="Bad sequence of commands";break;
				case 504:	msg="Command parameter not implemented";break;
				case 550:	msg="Requested action not taken: mailbox unavailable (e.g., mailbox not found, no access, or command rejected for policy reasons)";break;
				case 551:	msg="User not local; please try <forward-path> (See section 3.4)";break;
				case 552:	msg="Requested mail action aborted: exceeded storage allocation";break;
				case 553:	msg="Requested action not taken: mailbox name not allowed (e.g., mailbox syntax incorrect)";break;
				case 554:	msg="Transaction failed  (Or, in the case of a connection-opening response, \"No SMTP service here\")";break;
			}
			break;
	}
	if(*pref=='?' || !(*pref))
		sprintf(tmsg,txt,pref,nr);
	else
		sprintf(tmsg,txt,pref,nr&0xfff);
	logPrintf(2,TRUE,tmsg);
	if(msg)
	{

		while(strlen(msg)>70)
		{
			char *ptr;
			strncpy(tmsg,msg,70);
			tmsg[70]='\0';
			ptr=strrchr(tmsg,' ');
			if(ptr)
				strcpy(ptr,"\n");
			else
				strcat(ptr,"\n");
			msg+=strlen(tmsg)-1;
			logPrintf(2,FALSE,tmsg);
		}
		logPrintf(2,FALSE,"%s\n",msg);
	}
}